
// Demo: struct item alignment and padding
// Bytes: (integers=FF, chars='1'..., padding=00)
// Try (on 64bit system):
//   gcc        thisfile.c
//   gcc -m32   thisfile.c

#include <stdio.h>

// define two structures with the same fields
struct Test1 {
    char c1;
    short s;
    char c2;
    int i;
    char c3;
    long long l;
    char c4;
};

#pragma pack(1)         // without field alignment (slow)
struct Test2 {
    char c1;
    short s;
    char c2;
    int i;
    char c3;
    long long l;
    char c4;
};

// define two (incompatible) objects with the same field values
struct Test1 t1 = { '1', -1, '2', -1, '3', -1LL, '4' };
struct Test2 t2 = { '1', -1, '2', -1, '3', -1LL, '4' };

void dump(const void *addr, const size_t sz) {
    const unsigned char *p = addr;
    printf("memory[%zd] = {\n  ", sz);
    for (int i = 0; i < sz; ++i)
        printf("%02x,", p[i]);  // bytes
    printf("\n/*");
    for (int i = 0; i < sz; ++i)
        printf("%2d ", i);      // index for readability
    printf("*/\n}\n");
}

int main(void) {
    printf("struct Test1:\n");
    dump(&t1, sizeof(struct Test1));
    printf("struct Test2(packed):\n");
    dump(&t2, sizeof(struct Test2));
    return 0;
}
